package com.displaynone.acss.components.auth.models.user.repository

import androidx.paging.PagingSource
import androidx.paging.PagingState
import com.displaynone.acss.components.auth.models.user.repository.dto.VisitDto

class VisitListPagingSource(
    private val request: suspend (pageNum: Int, pageSize: Int) -> Result<List<VisitDto>>
) : PagingSource<Int, VisitDto>() {
    override fun getRefreshKey(state: PagingState<Int, VisitDto>): Int? {
        return state.anchorPosition?.let {
            state.closestPageToPosition(it)?.prevKey?.plus(1)
                ?: state.closestPageToPosition(it)?.nextKey?.minus(1)
        }
    }

    override suspend fun load(params: LoadParams<Int>): LoadResult<Int, VisitDto> {
        val pageNum = params.key ?: 0
        return request.invoke(
            pageNum,
            params.loadSize
        ).fold(
            onSuccess = { value ->
                LoadResult.Page(
                    data = value,
                    prevKey = (pageNum - 1).takeIf { it >= 0 },
                    nextKey = (pageNum + 1).takeIf { value.size == params.loadSize }
                )
            },
            onFailure = { error ->
                LoadResult.Error(error)
            }
        )
    }
}